unit example3;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, formula;

type
  TForm1 = class(TForm)
    Formula1: TFormula;
    Button1: TButton;
    Button2: TButton;
    procedure Button1Click(Sender: TObject);
    procedure Button2Click(Sender: TObject);
  private
    { Private-Deklarationen }
    PopValue : TPopValueFunction; // This function returns the topmost value on the stack
    PushValue: TPushValueProcedure; // This procedure pushes a value on the stack
    procedure NewFunction;          // This is the new function
                                    // note : It has to be defined inside an object
  public
    { Public-Deklarationen }
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}

procedure TForm1.NewFunction;
var a1,a2,a3 : Extended;
begin
   a3:=PopValue;
   a2:=PopValue;
   a1:=PopValue;
   PushValue(a1-a2-a3);
end;

procedure TForm1.Button1Click(Sender: TObject);
begin
   // This example shows how to add new functions to TFormula without descending
   // from TFormula
   // note : functions added in this manner are only valid in this instance of Tformula
   MessageDlg('We now add a new function called NewFuncName which expects 3 parameters'+#13#10+
              'a1, a2 and a3. It returns the result of a1-a2-a3.',mtInformation,[mbok],0);
   Formula1.AddFunction('NewFuncName',3,NewFunction,PopValue,PushValue);
   // explanation :
   // 1.parameter : The name of the function to be used in the formula string
   // 2.parameter : The number of parameters the function expects
   // 3.parameter : The new function itself
   //               This new function has to be defined in an object (e.g. TForm )
   // 4.parameter : Function to return the topmost value on the stack
   //               var PopValue : TPopValueFunction;
   //               TPopValueFunction is defined as
   //               type TPopValueFunction = function:Extended of object;
   // 5.parameter : Procedure to push a value on the stack
   //               var PushValue : TPushValueProcedure;
   //               TPushValueProcedure is defined as
   //               type TPushValueProcedure = procedure(Value:Extended) of object;

   Formula1.Formula:='NewFuncName(5,2,1)';
   MessageDlg('NewFuncName(5,2,1) = '+FloatToStrF(Formula1.Calculate2,ffGeneral,20,2),mtInformation,[mbok],0);
end;



procedure TForm1.Button2Click(Sender: TObject);
var i : byte;
    txt : String;
begin
   MessageDlg('We now add the new constants e and sqrt2.'+#13#10+
              'e is e=exp(1)'+#13#10+
              'sqrt2 is sqrt2=sqrt(2)',mtInformation,[mbok],0);
   // Clear Constants
   Formula1.Constants.Clear;
   Formula1.Constants.Add('e',exp(1),'exp(1)');
   Formula1.Constants.Add('sqrt2',sqrt(2),'sqrt(2)');
   // TConstantsList.Add(Name:String;Value:Extended;Formula:String);
   // Name : Name to be used in formulas
   // Value: associated Value
   // Formula: Formula to calculate the Value (optional, only used in the property editor)


   // Now use these constants
   Formula1.Formula:='sqrt2^(e*x)';

   MessageDlg('We now calculate sqrt2^(e*x) for several values of x.',mtInformation,[mbok],0);

   txt:='f(x) = sqrt2 ^ (e*x)'#13#10#13#10;
   For i:=1 to 4 do
   begin
      txt:=txt+'f('+intToStr(i)+') = ' + FloatToStrF(Formula1.Calculate([i]),ffGeneral,20,10)+#13#10;
   end;
   MessageDlg(txt,mtInformation,[mbok],0);
end;

end.
